/*******************************************************/
/*       Aquariumsteuerung incl. EVG-Ansteuerung       */
/*   gesteuert mit Hilfe einer 2-Kanal Zeitschaltuhr   */
/*******************************************************/



#define F_CPU 8000000UL

#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include <avr/wdt.h>


#define AQ_Port			PORTC
#define Lampe 			PC7
#define Wasserheizung 	PC6
#define Bodenheizung 	PC5
#define Luftpumpe 		PC4
#define CO2 			PC3

#define EVG 			PD6

//#define Uhr_Port		PINB


//#define UhrA_an 		PINB0					// Lampe, CO2 an/aus
//#define UhrA_aus 		PINB1

//#define UhrB_aus 		PINB3					// Aquarium ein/aus
//#define UhrB_an 		PINB2



volatile int a, b, u, x;
uint16_t c;



/* Initialisierung */
void init_uC (void)
{
	DDRA  = 0b11111111;
	PORTA = 0b11111111;
	DDRB  = 0b11110000;
	PORTB = 0b00000000;
	DDRC  = 0b11111111;
	PORTC = 0b00000111;
	DDRD  = 0b11111111;
	PORTD = 0b00000000;
	WDTCR = 0b00000111;
	wdt_disable();
}

void init_pwm (void) 
{        
	TCCR0 |= (1<<CS00);
	TCCR1A|= (1<<COM1A1)|(1<<COM1A0)|(1<<WGM11)|(1<<WGM10); 
  	TCCR1B|= (1<<CS10);
	OCR1A = 0;
}

/* Zeitverzgerung */

void delay_sek (uint16_t sek)
{
	for (a=0;a<sek;a++)
	{
		for (b=0;b<1000;b++)
		{
			_delay_ms(1);
		}
	}
}

int Aus (void)
{
	AQ_Port &= ~(1 << CO2);						// Uhr B aus
	AQ_Port &= ~(1 << Wasserheizung);
	AQ_Port &= ~(1 << Bodenheizung);
	AQ_Port &= ~(1 << Luftpumpe);
	OCR1A = 0;
	AQ_Port &=~(1 << Lampe);
	u = 0;
	u += PINB & (1<<PB3);
	u <<= 1;
	u += PINB & (1<<PB2);
	u <<= 1;
	u += PINB & (1<<PB1);
	u <<= 1;
	u += PINB & (1<<PB0);
	wdt_reset();
	return(1);
}

int Pause (void)
{
	AQ_Port |= (1 << CO2);						// Uhr A aus & Uhr B an
	AQ_Port &=~(1 << Wasserheizung);
	AQ_Port &=~(1 << Bodenheizung);
	AQ_Port &=~(1 << Luftpumpe);
	if (OCR1A == 1023)
	{
		for (c=1023; c>500; c--)
		{
			OCR1A = c;
			delay_sek (1);
			wdt_reset();
		}
	OCR1A = 500;
	AQ_Port &=~(1 << Lampe);
	}
	if (OCR1A == 0)
	{
		OCR1A = 500;
	}
	u = 0;
	u += PINB & (1<<PB3);
	u <<= 1;
	u += PINB & (1<<PB2);
	u <<= 1;
	u += PINB & (1<<PB1);
	u <<= 1;
	u += PINB & (1<<PB0);
	return(1);
}

int An (void)
{
	AQ_Port |= (1 << CO2);						// Uhr A an & Uhr B an
	AQ_Port |= (1 << Wasserheizung);
	AQ_Port |= (1 << Bodenheizung);
	AQ_Port |= (1 << Luftpumpe);
	AQ_Port |= (1 << Lampe);
	if (OCR1A == 500)
	{
		for (c=500; c<1023; c++)
		{
			OCR1A = c;
			delay_sek (1);
			wdt_reset();
		}
	OCR1A = 1023;
	}
	u = 0;
	u += PINB & (1<<PB3);
	u <<= 1;
	u += PINB & (1<<PB2);
	u <<= 1;
	u += PINB & (1<<PB1);
	u <<= 1;
	u += PINB & (1<<PB0);
	return(1);
}



/* Hauptprogramm */

int main (void)
{
	init_uC();
	init_pwm();
	wdt_enable(7);
	x += PINB & (1<<PB3);
	x <<= 1;
	x += PINB & (1<<PB2);
	x <<= 1;
	x += PINB & (1<<PB1);
	x <<= 1;
	x += PINB & (1<<PB0);
	wdt_reset();
Uhr:
	while  (u == x)
	{
		x = 0;
		x += PINB & (1<<PB3);
		x <<= 1;
		x += PINB & (1<<PB2);
		x <<= 1;
		x += PINB & (1<<PB1);
		x <<= 1;
		x += PINB & (1<<PB0);
		wdt_reset();
	}
	if (u != x)
	{
		_delay_ms(25);
		x = 0;
		x += PINB & (1<<PB3);
		x <<= 1;
		x += PINB & (1<<PB2);
		x <<= 1;
		x += PINB & (1<<PB1);
		x <<= 1;
		x += PINB & (1<<PB0);
		wdt_reset();
		if (u != x)
		{
			if (x == 68)
			{
				Aus();
			}
			wdt_reset();
			if (x == 20)
			{
				Pause();
			}
			wdt_reset();
			if (x == 17)
			{
				An();
			}
			wdt_reset();
			x = 0;
			x += PINB & (1<<PB3);
			x <<= 1;
			x += PINB & (1<<PB2);
			x <<= 1;
			x += PINB & (1<<PB1);
			x <<= 1;
			x += PINB & (1<<PB0);
			wdt_reset();
		}
	}
	goto Uhr;
}
